PROC GLOBAL
set explicit;
{ Working variables -- check the lengths needed for all fields }
alpha(32) CSPro;						{ CSPro executables directory }
alpha(99) proj;							{ project directory name }
alpha(8)  cmapp, deapp, edapp, exapp, gpapp, scapp, dic, deappB; { application names and dictionary name }
alpha(16) clusterapp;
alpha(20) printapp;
alpha(8)  opid;							{ operator ID }
alpha(2)  datakey;                      { data key dirve letter, e.g. "E:" }
alpha(2)  wrkdrv;               		{ workdrive letters, e.g. "C:" }
alpha(2)  netdrv;               		{ network data drive letters, e.g. "F:" }
alpha(99) wrkprj, backup, expdir, gpsx, final, raw, veri, tempdir;     { directories }
alpha(99) netprj, netdat, netver;     	{ network directories }
alpha(99) dicts, entry, super;          { directory names for DICTS, ENTRY, SUPER -- suggest using these names }
alpha(3)  xcluster;                     { cluster number -- make sure length is correct!!! }
alpha(2)  fcodes;                       { file codes - M = Main, V = Verification }
alpha(1)  frstrun;						{ is this the first run of the editing program }
alpha(99) wdata, mdata, vdata;			{ Name of raw data file, main entry data file and verification data file }
alpha(99) fname, bchname;      			{ auxiliary filenames for main data and bchfile }
alpha(250) cmdline;

numeric i, always, choice, chkok, noyes, err, docopy, maxclusternumber, startpos,
        dd, dr, de, ss, valid, structok, keychoice, mainver, endfile, verified;

file pfffile jstone state diffile;

{ ---------------------------------------------------------------------------- }

function chkdir(alpha(99) xdir)
  if fileexist(xdir) <> 1 then
    {+EN} errmsg("Directory %s does not exist",xdir);    {EN+}
    {{ES} errmsg("Este directorio %s no existe", xdir ); {ES}}
    {{FR} errmsg("Directoire %s n'existe pas",xdir);     {FR}}
    stop(-1);
  endif;
end;

function chkexist(mychoice)
  chkok = 1;
  if mychoice in 5,6,8,9,10 then { using raw data }
    if !fileexist(wdata) then { main data doesn't exist in raw directory }
      {+EN} errmsg("Raw data file for cluster %s does not exist",xcluster);              {EN+}
      {{ES} errmsg("El archivo de datos principal del conglomerado %s ?????? [add spanish",xcluster); {ES}}
      {{FR} errmsg("Fichier de donnees brutes de la grappe %s n'existe pas",xcluster);   {FR}}
      chkok = 0;
    endif;
  endif;
  if chkok & mychoice = 5 then { verifying data }
    { !!! NETWORK !!! change wording of message if using a network }
     vdata = concat(strip(netver), "\", fcodes[2:1], xcluster, ".DAT");
     if !fileexist(vdata) then { verification data doesn't exist }
       { !!! NETWORK !!! change wording of message if using a network }
       {+EN} errmsg("Verification data file for cluster %s is not on the network",xcluster);            {EN+}
       {{ES} errmsg("Archivo principal de datos del conglomerado %s no existe en el network",xcluster); {ES}}
       {{FR} errmsg("Verification donnees pour la grappe %s n'est pas dans le reseau",xcluster);        {FR}}
      chkok = 0;
    endif;
  endif;
  chkexist = chkok;
end;

function init_tracking();
   if accept
      {+EN} ("Initialize cluster tracking file","Yes","No") = 1        {EN+} 
      {{ES} ("ES") = 1                                                 {ES}}
      {{FR} ("Initialiser fichier de controle grappe","Oui","Non") = 1 {FR}}
   then
       {+EN} display("Ready to initialize. Please click OK, then wait");               {EN+}
       {{ES} display("ES");                                                            {ES}}
       {{FR} display("Pret pour initialiser. S'il vous plait, cliquer OK et attend");  {FR}}
   
    { set whole record to zero }
    TRECV       = 0; 
    TNUMHHSEL   = 0;  { !!! if expected # of HHs will be the same for each cluster, assign it here }
    TNUMHHQUEST = 0; TNUMHHCOMP = 0; 
    TNUMWMQUEST = 0; TNUMWMCOMP = 0; 
    TNUMMMQUEST = 0; TNUMMMCOMP = 0; 
    TNUMCHQUEST = 0; TNUMCHCOMP = 0;
    TENTRYOPID  = 0; TENTRY  = 0; 
    TVENTRYOPID = 0; TVENTRY = 0; 
    TCHECK = 0; TVERI = 0; TRAW = 0; TEDIT = 0; TFINAL = 0;
    { For each cluster }
    open(cluster_tracking_dict);
    do i = 1 while i <= maxclusternumber
      { !!! CLUSTER !!! check valid cluster numbers }
      recode i => valid;
         1-800 => 1;
           	 => 0;
      endrecode;
      if valid then
        tcluster = i;
        { Write information for this cluster to file }
        if !writecase(cluster_tracking_dict, tcluster) then
          {+EN} errmsg("Not able to create cluster tracking data file");              {EN+}
          {{ES} errmsg("No fue posible crear el archivo de datos del conglomerado");  {ES}}
          {{FR} errmsg("Pas possible de creer les donnees de controle de la grappe"); {FR}}
          break;
        endif;
      endif;
    enddo;
    { reset value of tcluster---which may have changed above---to equal ecluster }
    tcluster = ecluster;
    close(cluster_tracking_dict);
  endif;
end;

{ Updates the cluster tracking file }
function update_tracking(update_field);
  { Write cluster tracking info }
  tcluster = ecluster;
  if loadcase(cluster_tracking_dict, tcluster) then
	if     update_field = 1 then TCHECK = sysdate("DDMMYYYY");
	elseif update_field = 2 then TVERI  = sysdate("DDMMYYYY");
	elseif update_field = 3 then TRAW   = sysdate("DDMMYYYY");
	elseif update_field = 4 then TEDIT  = sysdate("DDMMYYYY");
	elseif update_field = 5 then TFINAL = sysdate("DDMMYYYY");
	endif;
	writecase(cluster_tracking_dict, tcluster);
	close(cluster_tracking_dict);
  else
	{+EN} errmsg("Cluster not found in tracking file");            {EN+}
	{{ES} errmsg("ES");                                            {ES}}
	{{FR} errmsg("Grappe non trouve dans le fichier de controle"); {FR}}
  endif;
end;

function createbch(alpha(99) xname );
  setfile(pfffile, strip(xname));
  filecreate(pfffile);
  filewrite(pfffile, "[CSPro Application]");
  filewrite(pfffile, "Version=CSPro 4.1");
  filewrite(pfffile, "Label=MICS4");
  filewrite(pfffile, "Name=MICS4");
  filewrite(pfffile, "AppType=Batch");
  filewrite(pfffile, "[AppCode]");
  filewrite(pfffile, concat("File=1/1/2010 0:00:00 AM,", strip(entry), "\", strip(deapp), ".app" ) );
  filewrite(pfffile, "[Message]");
  filewrite(pfffile, concat("File=1/1/2010 0:00:00 AM,", strip(entry), "\", strip(deapp), ".mgf" ) );
  filewrite(pfffile, "[Order]");
  filewrite(pfffile, concat("File=1/1/2010 0:00:00 AM,", strip(entry), "\", strip(deapp), ".fmf" ));
  filewrite(pfffile, "[Dictionary Types]");
  filewrite(pfffile, concat("Dict-Type=", strip(dicts), "\", strip(dic), ".dcf,Input,", strip(Entry), "\", strip(deapp), ".fmf" ) );
  close(pfffile);
  bchname = xname;
end;

function delb( );
 if FileExist (bchname) then filedelete (bchname);
 endif;
end;

function nodiff( alpha(255) differences );
  SetFile( diffile, differences );
  endfile = 0;
  if !fileexist(diffile) then endfile = 2 endif;
  { also using cmdline as a string var to read the file into }
  while !endfile do
    if FileRead( diffile,  cmdline ) < 1 then { end of file found }
      endfile = 2;
    elseif pos( "[", cmdline ) then        { Found the start of an ID string - means there must be some differences }
      endfile = 1;
    endif;
  enddo;
  close( diffile );
  nodiff = ( endfile =  2 );
end;

{ Initalizes the first part of the PFF file for genpff below }
function initPFF( alpha(8) xtype, alpha(99) xname );
  setfile(pfffile, strip(xname));
  filecreate(pfffile);
  filewrite(pfffile, "[Run Information]");
  filewrite(pfffile, "Version=CSPro 4.1");
  filewrite(pfffile, concat("AppType=", strip(xtype)));
  filewrite(pfffile, "");
end;

function genpff(mychoice)
  if mychoice = 3 then { structure check }
    initPFF("Batch", concat(strip(super), "\", strip(scapp), ".pff"));
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, concat("Application=.\", strip(scapp), ".bch"));
    filewrite(pfffile, concat("InputData=", strip(wdata)));
    filewrite(pfffile, concat("Listing=.\", strip(scapp), ".lst"));
    filewrite(pfffile, "");
    filewrite(pfffile, "[ExternalFiles]");
    filewrite(pfffile, "CLUSTER_TRACKING_DICT=.\cluster_tracking.dat");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, "ViewListing=Always");
  elseif mychoice = 4 then { entry in batch }
    createbch(concat(strip(super), "\", strip(deappB), ".bch"));
    initPFF("Batch", concat(strip(super), "\", strip(deappB), ".pff"));
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, concat("Application=.\", strip(deappB), ".bch"));
    filewrite(pfffile, concat("InputData=", strip(wdata)));
    filewrite(pfffile, concat("Listing=.\", strip(deappB), ".lst"));
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, concat("Parameter=", xcluster));
    filewrite(pfffile, "ViewListing=Always");
    filewrite(pfffile, "SkipStructure=Yes");
    filewrite(pfffile, "CheckRanges=Yes");
    filewrite(pfffile, "ViewResults=Yes");
  elseif mychoice = 5 then { verification }
    initPFF("Compare", concat(strip(super), "\", strip(cmapp), ".pff"));
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, "Application=.\compare.cmp");
    filewrite(pfffile, concat("InputData=", strip(wdata)));
    filewrite(pfffile, "Listing==C:\MICS4\CSPRO\SUPER\compare.cmp.lst");
    filewrite(pfffile, concat("ReferenceData=", strip(netver), "\", fcodes[2:1], xcluster, ".DAT"));
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, "ViewListing=Always");
  elseif mychoice = 8 then { editing }
  	noyes = accept
      {+EN} ("Do you want to see warning messages [type M] for the cluster?","No","Yes");     {EN+}
      {{ES} ("ES");                                                                           {ES}}
      {{FR} ("Voulez vous voir les messages d'attention [type M] de la grappe?","Non","Oui"); {FR}}
  	frstrun = edit("9",noyes-1);
    initPFF("Batch", concat(strip(super), "\", strip(edapp), ".pff"));
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, "Application=.\editing.bch");
    filewrite(pfffile, concat("InputData=", strip(wdata)));
    filewrite(pfffile, "Listing=.\editing.lst");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, "ViewListing=Always");
    filewrite(pfffile, concat("Parameter=",frstrun));
  elseif mychoice = 9 then { data entry }
    initPFF("Entry", concat(strip(entry), "\", strip(deapp), ".pff"));
    filewrite(pfffile, "[DataEntryInit]");
    filewrite(pfffile, concat("OperatorID=", strip(opid)));
    filewrite(pfffile, "Lock=Add, Verify");
    filewrite(pfffile, "Fullscreen=No");
    filewrite(pfffile, "NoFileOpen=Yes");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, concat("Application=", strip(entry), "\", strip(deapp), ".enc"));
    filewrite(pfffile, concat("InputData=", strip(wdata)));
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, concat("Parameter=", xcluster));
  elseif mychoice = 12 then { export to SPSS }
    initPFF("Batch", concat(strip(expdir), "\", strip(exapp), ".pff"));
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, "Application=..\export\export.bch");
    filewrite(pfffile, concat("InputData=", strip(final), "\all.dat"));
    filewrite(pfffile, "Listing=..\export\export.lst");
    filewrite(pfffile, "");
    filewrite(pfffile, "[ExternalFiles]");
    filewrite(pfffile, "WHOLMS=..\export\WHOlms.dat");
    filewrite(pfffile, "");  
    filewrite(pfffile, "[UserFiles]");
    filewrite(pfffile, "MYHH=..\..\SPSS\MYHH.DAT");
    filewrite(pfffile, "MYHL=..\..\SPSS\MYHL.DAT");
    filewrite(pfffile, "MYTN=..\..\SPSS\MYTN.DAT");
    filewrite(pfffile, "MYWM=..\..\SPSS\MYWM.DAT");
    filewrite(pfffile, "MYFG=..\..\SPSS\MYFG.DAT");
    filewrite(pfffile, "MYBH=..\..\SPSS\MYBH.DAT");
    filewrite(pfffile, "MYMM=..\..\SPSS\MYMM.DAT");
    filewrite(pfffile, "MYCH=..\..\SPSS\MYCH.DAT");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, "ViewListing=Always");
  elseif mychoice in 14,15 then { enter or modify GPS data }
    initPFF("Entry", concat(strip(gpsx), "\", strip(gpapp), ".pff"));
    filewrite(pfffile, "[DataEntryInit]");
    filewrite(pfffile, concat("OperatorID=", strip(opid)));
    if mychoice = 14 then { add data }
      filewrite(pfffile, "Lock=Modify,Verify");
    elseif mychoice = 15 then { modify data }
      filewrite(pfffile, "Lock=Add,Verify");
    endif;
    filewrite(pfffile, "Fullscreen=No");
    filewrite(pfffile, "NoFileOpen=Yes");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, concat("Application=", strip(gpsx), "\", strip(gpapp), ".ent"));
    filewrite(pfffile, concat("InputData=",   strip(gpsx), "\gps.dat"));
    filewrite(pfffile, "");
  elseif mychoice = 18 then { print cluster tracking info check }
    initPFF("Batch", concat(strip(super), "\", strip(printapp), ".pff"));
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, "Application=.\print_tracking_info.bch");
    filewrite(pfffile, concat("InputData=", strip(super), "\", strip(clusterapp), ".DAT"));
    filewrite(pfffile, "Listing=.\print_tracking_info.lst");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, "ViewListing=Always");
  elseif mychoice = 19 then { field check tables }
    initPFF("Batch", concat(strip(super), "\cfldck.pff"));
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, "Application=.\cfldck.bch");
    filewrite(pfffile, concat("InputData=", strip(raw), "\ALL.DAT"));
    filewrite(pfffile, "Listing=.\cfldck.lst");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, "ViewListing=Always");
  endif;

  close(pfffile);
end;

function Execapp( alpha(8) xtype, alpha(99) xdir, alpha(20) xappname );
  Execsystem(concat('"', strip(cspro), '\CS', strip(xtype), '.exe" "',
                    strip(xdir), "\", strip(xappname), '.pff"'),
             maximized,wait);
end;

function Textview( alpha(99) xdir, alpha(12) xname );
  Execsystem(concat('"', strip(cspro), '\TextView.exe" "', strip(xdir), '\', strip(xname), '"'), maximized,wait);
end;

function runpff(mychoice);
  if     mychoice =  1 then { cluster info entry } Execapp( "Entry", super , clusterapp );
  elseif mychoice =  3 then { structure checking } Execapp( "Batch", super , scapp      );
    structok = 0;
    if accept
      {+EN} ("Structure check with no errors?", "No",  "Yes" ) = 2   {EN+}
      {{ES} ("ES",                              "No",  "Si"  ) = 2   {ES}}
      {{FR} ("Controle de la structure complete?","Non", "Oui" ) = 2 {FR}}
      then structok = ecluster endif;
    
  elseif mychoice =  4 then { entry in batch     } 
    if structok <> ecluster then
      {+EN} errmsg("Run questionnaire structure check for cluster %03d first before checking skip and error messages", ecluster); {EN+}
      {{ES} errmsg("ES", ecluster);                                                                                               {ES}}
      {{FR} errmsg("Lancer le programme de controle de structure pour la grappe %03d avant de controler les sauts et les messages d'erreurs", ecluster); {FR}}
    else                                           Execapp( "Batch", super , deappb     );
      if accept(
          {+EN} "Entry-in-batch with no more errors to correct?", "No" , "Yes"       {EN+}
          {{ES} "ES", "No" , "Si"                                                    {ES}}
          {{FR} "Saisie-en-batch avec pas plus d'erreurs de corriger?", "Non", "Oui" {FR}}
               ) = 2 then  update_tracking(1 {TCHECK}); endif;
    endif;
  elseif mychoice =  5 then { verification       } Execapp( "Diff",  super , cmapp      );
    fname = "C:\MICS4\CSPRO\SUPER\compare.cmp.lst";
    verified = nodiff( fname );
    if verified then
      {+EN} errmsg( "No differences found" );            {EN+}
      {{ES} errmsg( "No diferencias encontradas" );      {ES}}
      {{FR} errmsg( "Aucune diffrence n'est trouve" ); {FR}}
    { if accept(
        {+EN} "Verification with no errors?", "No" , "Yes"      {EN+}
        {{ES} "ES?", "No" , "Si"                                {ES}}
        {{FR} "Verification avec pas d'erreurs?", "Non" , "Oui" {FR}}
             ) = 2 then } update_tracking(2 {TVERI}); { endif; }
    else
      {+EN} errmsg( "Differences found - please correct the main and verification files" );        {EN+}
      {{ES} errmsg( "Diferencias encontradas - ???? [add ES]" );                                   {ES}}
      {{FR} errmsg( "Diffrences trouves - corriger les fichier principale et de verification" ); {FR}}
    endif;
  elseif mychoice =  8 then { editing            } 
    if TRAWY = 0 then
      {+EN} errmsg("Backup raw data for cluster %03d first running the secondary editing ", ecluster);                             {EN+}
      {{ES} errmsg("ES", ecluster);                                                                                                {ES}}
      {{FR} errmsg("Sauvegarder le fichier brut pour la grappe %03d avant de lancer le programme d'edition secondaire", ecluster); {FR}}
    else                                           Execapp( "Batch", super , edapp      );
	  if accept(
          {+EN} "Editing with no more errors to correct?", "No" , "Yes"         {EN+}
          {{ES} "ES", "No" , "Si"                                               {ES}}
          {{FR} "L'edition avec pas d'autres erreurs a corriger?", "Non", "Oui" {FR}}
               ) = 2 then update_tracking(4 {TEDIT});  
      endif;
    endif;
  elseif mychoice =  9 then { data entry         } Execapp( "Entry", entry , deapp      );
  elseif mychoice = 12 then { export to SPSS     } Execapp( "Batch", expdir, exapp      );
  elseif mychoice = 14 or
         mychoice = 15 then { GPS data entry     } Execapp( "Entry", gpsx,    gpapp      );
  elseif mychoice = 18 then
                   { print cluster tracking info } Execapp( "Batch", super,  printapp   );
  elseif mychoice = 19 then { field check tables } Execapp( "Batch", super, "cfldck"     ); 
         filedelete("cfldck.rtf");
         Execsystem("c:\mics4\cspro\tableseditor\tables.exe cfldck.tbd",maximized,wait);
         Execsystem(concat("C:\Program Files\Microsoft Office\Office12\winword.exe", "cfldck.rtf"),maximized,wait);
  endif;
end;

function csrun(mychoice)
  if chkexist(mychoice) then
    genpff(mychoice);
    runpff(mychoice);
  endif;
end;

function transfer(mychoice)
  chkok = 1;
  if mychoice in 3 then { checking structure }
    if !fileexist(wdata) or accept(
                                   {+EN} "Recopy the main data file?","No","Yes"              {EN+}
                                   {{ES} "ES","No","Si"                                       {ES}}
                                   {{FR} "Recopier le fichier saisie principale?","Non","Oui" {FR}}
                                  ) = 2 then { wdata = local comp's \raw folder }
      mdata = concat(strip(netdat), "\", fcodes[1:1], xcluster, ".DAT"); 		{ mdata now looking at network's folder }
      if !fileexist(mdata) then 											{ if main data not on network }
        {+EN} errmsg("Main data file for cluster %s does not exist on the network",xcluster); {EN+}	{ complain, file ! on supervisor's computer or network }
        {{ES} errmsg("ES",xcluster);                                                                    {ES}}
        {{FR} errmsg("Fichier saisie principale de la grappe %s n'existe pas dans le reseau",xcluster); {FR}}
		chkok = 0;
      else
        filecopy(mdata, wdata);			{ o/w, ok to copy file from ntwk to local comp }
        {+EN} errmsg("Main data file for cluster %s copied to raw directory",xcluster);                   {EN+}
        {{ES} errmsg("ES",xcluster);                                                                      {ES}}
        {{FR} errmsg("Fichier saisie principale de la grappe %s copie dans le directoire brut",xcluster); {FR}}
      endif;
	endif;
    if chkok then	{ now finally run the structure check! }
      csrun(mychoice);
	endif;

  elseif mychoice = 5 then		{ Verification }
    if chkexist(mychoice) then
      if TCHECKY = 0 then
        {+EN} errmsg("Run structure check for cluster %03d first before verification", ecluster); {EN+}
        {{ES} errmsg("ES", ecluster);                                                             {ES}}
        {{FR} errmsg("Lancer le programme de control de la structure pour la grappe %03d avant du verification", ecluster); {FR}}
      else
   		genpff(mychoice);
   		runpff(mychoice);
     endif;
   endif;	
  elseif mychoice = 6 then
    if chkexist(mychoice) then
      if TVERIY = 0 then
        {+EN} errmsg("Run verification for cluster %03d first before copying the data to backup directory", ecluster); {EN+}
        {{ES} errmsg("ES", ecluster);                                                                                  {ES}}
        {{FR} errmsg("Lancer le programme de verification pour la grappe %03d avant de copier le fichier dans le directoire backup", ecluster); {FR}}
      else
        { copy the data to the backup directory }
        filecopy(wdata, concat(strip(backup), "\", fcodes[1:1], xcluster, ".DAT"));
        {+EN} errmsg("Main data file for cluster %s copied to backup directory",xcluster);                  {EN+}
        {{ES} errmsg("ES",xcluster);                                                                        {ES}}
        {{FR} errmsg("Fichier saisie principale de la grappe %s copie dans le directoire backup",xcluster); {FR}}
	    update_tracking(3 {TRAW});
	   endif; 
    endif;
  elseif mychoice = 10 then
    if chkexist(mychoice) then
      if TEDITY = 0 then
        {+EN} errmsg("Run secondary editiong for cluster %03d first before copying the data to final directory", ecluster);              {EN+}
        {{ES} errmsg("ES", ecluster);                                                                                                    {ES}}
        {{FR} errmsg("Lancer le programme l'edition pour la grappe %03d avant de copier le fichier dans le directoire final", ecluster); {FR}}
      else
        { copy the data to the final directory }
        filecopy(wdata, concat(strip(final), "\", fcodes[1:1], xcluster, ".DAT"));
        {+EN} errmsg("Main data file for cluster %s copied to final directory",xcluster);                   {EN+}
        {{ES} errmsg("ES",xcluster);                                                                        {ES}}
        {{FR} errmsg("Fichier saisie principale de la grappe %s copie dans le directoire final ",xcluster); {FR}}
	    update_tracking(5 {TFINAL});
	  endif;
    endif;
  elseif mychoice = 12 then
    { concatenate the cluster data in final }
    fileconcat(concat(strip(final), "\all.dat"), concat(strip(final),"\",fcodes[1:1],"*.dat"));
    csrun(mychoice);
  elseif mychoice = 19 then
    { concatenate the cluster data in raw }
    fileconcat(concat(strip(raw), "\all.dat"), concat(strip(raw),"\",fcodes[1:1],"*.dat"));
    csrun(mychoice);
    
  elseif mychoice in 20,21 then
    keychoice = accept(
{+EN}     "Insert data key and select drive",					{EN+}
{{ES}     "Coloque clave de datos y selecionar el puerto",		{ES}}
{{FR}     "Insrez le cl de donnes et selectionner l'unit",	{FR}}
          "E:","F:","G:","H:","I:");
    if keychoice then
      datakey = concat( "EFGHI"[keychoice:1],":" );
      mainver = mychoice - 19;
      fname = concat( fcodes[mainver:1], xcluster, ".dat" );
      if     mainver = 1 then tempdir = raw;
      elseif mainver = 2 then tempdir = veri;
      endif;
      if fileexist( concat(strip(tempdir), "\", strip(fname)) ) then
{+EN}   errmsg( "*** Cluster %s already exists on drive %s -- Please check",				{EN+}
{{ES}   errmsg( "*** El conglomerado % existe en el puerto %s -- Revise por favor",			{ES}}
{{FR}   errmsg( "*** Grappe %s existe dj dans le lecteur %s -- Veuillez vous en assurer",	{FR}}
                xcluster, strip(tempdir) );
      endif;
      filecopy( concat(strip(datakey), "\", strip(fname)), concat(strip(tempdir), "\", strip(fname)) );
{+EN} errmsg( "Copying data files for cluster %s from data key",					{EN+}
{{ES} errmsg( "Copiando los datos para el conglomerado %s de clave de datos",		{ES}}
{{FR} errmsg( "Copie des fichiers de donnes de la grappe %s du cl de donnes",	{FR}}
              xcluster );
      if fileexist( concat(strip(tempdir), "\", strip(fname)) ) then
{+EN}   errmsg( "*** Cluster %s copied from drive %s to %s",				{EN+}
{{ES}   errmsg( "*** El conglomerado % copiando de el puerto %s a %s",   	{ES}}
{{FR}   errmsg( "*** Grappe %s a copi de l'unit %s  %s",		        	{FR}}
                xcluster, strip(datakey), strip(tempdir) );
      else
{+EN}   errmsg( "*** Cluster %s not copied from drive %s",					{EN+}
{{ES}   errmsg( "*** El conglomerado % no esta copiando del puerto %s",	    {ES}}
{{FR}   errmsg( "*** Grappe %s ne copi pas de l'unit %s",				    {FR}}
                xcluster, strip(datakey) );
      endif;
    endif;    
    
  endif;
end;

PROC SUPER_FF
preproc

{ The following directories are assumed }
{ \MICS4        Main survey directory (some files located here)}
{ \MICS4\BACKUP Directory used to store backup data            }
{ \MICS4\DICTS  Directory used to store dictionaries           }
{ \MICS4\ENTRY  Data entry programs located here               }
{ \MICS4\EXPORT Directory used to store export programs        }
{ \MICS4\FINAL  Directory used to store final data             }
{ \MICS4\GPS    Directory used to store GPS data               }
{ \MICS4\RAW    Directory used to store raw data               }
{ \MICS4\SUPER  Directory for supervisor programs              }

CSPro  = "C:\Program Files\CSPro 4.1"; { CSPRO directory }
proj   = "MICS4\CSPRO";	  			{ Project directory }
cmapp  = "COMPARE";					{ Compare application name }
deapp  = "ENTRY";					{ Entry application name }
edapp  = "EDITING";					{ Editing application name }
exapp  = "EXPORT";					{ Export application name }
gpapp  = "GPSENTRY";				{ GPS application name }
scapp  = "CHECK";					{ Structure check application name }
clusterapp = "CLUSTER_TRACKING";	{ Cluster info entry application name }
printapp   = "PRINT_TRACKING_INFO";	{ Cluster info printing application name }
deappB = "ENTRYB";					{ Entry application name for entry in batch }
dic    = "MICS4";					{ Dictionary name }
opid   = "MICS4"; 					{ this could be an ID set on each machine }
fcodes = "MV";						{ File codes - M = Main, V = Verification,) }
maxclusternumber = 800;				{ !!! CLUSTER !!! set value of largest cluster number }

wrkdrv = "C:";						{ Set work drive }
wrkprj = concat(strip(wrkdrv), "\", strip(proj));
raw    = concat(strip(wrkprj), "\RAW"   );	{ Raw data directory }
backup = concat(strip(wrkprj), "\BACKUP");	{ Backup of raw data directory }
final  = concat(strip(wrkprj), "\FINAL" );	{ Backup of final data directory }
dicts  = concat(strip(wrkprj), "\DICTS" );	{ Dictionary directory }
entry  = concat(strip(wrkprj), "\ENTRY" );	{ Entry applications directory }
expdir = concat(strip(wrkprj), "\EXPORT");	{ Export directory }
gpsx    = concat(strip(wrkprj), "\GPS"   );	{ GPS directory }
super  = concat(strip(wrkprj), "\SUPER" );	{ Supervisor's applications directory }

{ Check that all directories needed exist }
chkdir(wrkprj);
chkdir(dicts);
chkdir(entry);
chkdir(expdir);
chkdir(gpsx);
chkdir(super);
chkdir(backup);
chkdir(final);
chkdir(raw);

{ !!! NETWORK !!! Set network drive letter to the drive to use }
netdrv = "F:";
netprj = concat( strip(netdrv), "\", strip(proj) );		{ Can change this to a different structure if desired }
{ For testing - use C:\MICS4\NETWORK\CSPRO }
  netdrv = "C:";
  netprj = concat( strip(netdrv), "\MICS4\NETWORK" );
{ if no network, use local directories }
if !fileexist( strip(netprj) ) then
  {+EN} errmsg("Network directory %s does not exist. Using work directory %s", strip(netprj), strip(wrkprj));            {EN+}
  {{ES} errmsg("ES", strip(netprj), strip(wrkprj));                                                                      {ES}}
  {{FR} errmsg("Repertoire de reseau %s n'exist pas. Utilisant repertoire de travail %s", strip(netprj), strip(wrkprj)); {FR}}
  netdrv = wrkdrv;
  netprj = wrkprj;
endif;

{ Set network drive directories }
netdat = concat( strip(netprj), "\DATA" );
netver = concat( strip(netprj), "\VERI" );

{ check network directories exist }
if strip(netprj) <> strip(wrkprj) then
  { Check that all directories needed exist }
  chkdir(netprj);
  chkdir(netdat);
  chkdir(netver);
endif;

{ create the file name variable justone}
setfile(jstone,concat(strip(wrkprj),"\justone.txt"));

{ Check that there are not two concurrent sessions of CSPro loaded at the same time }
if fileexist(jstone) then
  errmsg(
    {+EN} "There is another session running or the computer crashed while running"                       {EN+}
    {{ES} "Hay otra sesin corriendo o el computador fall mientras corra"                              {ES}}
    {{FR} "Il y a une autre session en cours ou bien que l'ordinateur s'est plant en cours d'excution" {FR}}
        );
  if accept(
      {+EN} "Reset","Yes","No"            {EN+}
      {{ES} "Reiniciar","Si","No"         {ES}}
      {{FR} "Re-initialiser","Oui","Non"  {FR}}
           ) = 1 then
    filedelete(jstone)
  else
    stop(-1);
  endif;
endif;

filewrite( jstone,
     {+EN} "There is another session running or the computer crashed while running"                       {EN+}
     {{ES} "Hay otra sesin corriendo o el computador fall mientras corra"                              {ES}}
     {{FR} "Il y a une autre session en cours ou bien que l'ordinateur s'est plant en cours d'excution" {FR}}
         );
close( jstone );

PROC SUPER_LEVEL
preproc
{ Get the cluster, supervisor and interviewer numbers if already used before }
setfile( state, concat( strip(super), "\cluster.txt" ) );
if fileexist(state) then
  fileread( state, xcluster );
  ecluster = tonumber(xcluster);
endif;

PROC ECLUSTER
{ !!! CLUSTER !!! give list of valid cluster numbers }
recode ecluster => err;
        001-800 => 0;    { Give list of valid cluster numbers }
                => 1;
endrecode;
if err then
  {+EN} display("Cluster number not valid");    {EN+}
  {{ES} display("ES");                          {ES}}
  {{FR} display("Numero de grappe pas valide"); {FR}}
  reenter;
endif;

tcluster = ecluster;
if !fileexist(filename(cluster_tracking_dict)) | filesize(filename(cluster_tracking_dict)) = 0 then
  init_tracking();
elseif !loadcase(cluster_tracking_dict, tcluster) then
  {+EN} errmsg("Cluster %03d not found in tracking file",tcluster);            {EN+}
  {{ES} errmsg("Cluster %03d not found in tracking file",tcluster);            {ES}}
  {{FR} errmsg("Grappe %03d pas trouve dans le fichier de controle",tcluster); {FR}}
  reenter;
endif;

{ !!! CLUSTER !!! check length of edit pattern matches length of largest cluster number }
xcluster = edit("999",$);
setfile(state, concat( strip(super), "\cluster.txt" ), create);
filewrite(state, xcluster);

{ set location of raw data }
wdata = concat(strip(raw), "\", fcodes[1:1], xcluster, ".DAT");
{ set name of file that helps make sure that only one version of CSPRO runs at a time }
setfile(jstone, concat(strip(wrkprj),"\justone.txt"));

always = 1;
while always do   { loop forever }
  { Get the selected option }
  choice = accept(
  {+EN}
  				  "Supervisor menu options",
                  "T Enter cluster tracking information...........T",
                  "------------------------------------------------",
                  "A Check data structure.........................A",
                  "B Check data with entry-in-batch...............B",
                  "C Verify the data..............................C",
                  "D Backup the raw data..........................D",
                  "------------------------------------------------",
                  "E Run Secondary Editing Program................E",
                  "F Modify the data..............................F",
                  "G Backup the final data........................G",
                  "------------------------------------------------",
                  "H Export the data to SPSS (all clusters).......H",
                  "------------------------------------------------",
                  "I Enter GPS data...............................I",
                  "J Modify GPS data..............................J",
                  "------------------------------------------------",
                  "N New cluster number...........................N",
                  "V View cluster tracking information............V",
                  "P Field check tables...........................P",
                  "R Copy data file from operator's key...........R",
                  "S Copy verification file from operator's key...S",
                  "Q Quit.........................................Q");
  {EN+}     
  {{ES}           " [ADD ES] "
   {ES}}         
  {{FR}
                  "Options Menu Superviseur",
                  "T Entrer information controle grappe.............T",
                  "--------------------------------------------------",
                  "A Controler le structure des donnees.............A",
                  "B Controler les donnees avec saisie-en-batch.....B",
                  "C Verifier les donnees...........................C",
                  "D Sauvegarder les donnees brutes.................D",
                  "-------------------------------------------------",
                  "E Tourner le programme d'edition secondaire......E",
                  "F Modifier les donnees...........................F",
                  "G Sauvegarder les donnees finales................G",
                  "--------------------------------------------------",
                  "H Converter les donnees en SPSS (toutes grappes).H",
                  "--------------------------------------------------",
                  "I Saisir les donnees GPS.........................I",
                  "J Modifier les donnees GPS ......................J",
                  "--------------------------------------------------",
                  "N Nouveau numero grappe..........................N",
                  "V Revoir informations controle grappes...........V",
                  "P Tableau de contrle de terrain.................P", 
                  "R Copier fichier de donnees d'un agent de saisie.R",
                  "S Copier fichier de donnees de verification......S",
                  "Q Quitter........................................Q");
  {FR}}
  

  if choice = 1 then
    { Check what the user wants to do }
    startpos = accept(
                 {+EN} "Do you want to:",
                       "1 Enter receive date and cluster counts",
                       "2 Enter data operator ID",
                       "3 Enter verification data operator ID"); 
                 {EN+}
                 {{ES} "ES",
                       "ES",
                       "ES",
                       "ES"); 
                 {ES}}
                 {{FR} "Voulez-vous:", 
					   "1 Saisie date de reception et le nombre de grappes",
					   "2 Entrer code Agent de saisie", 
					   "3 Entrer code Agent verifieur"); 
			     {FR}}
    if startpos > 0 then { If user hits escape or cancel button, return to main menu }
      enter cluster_tracking_ff;
      { make SURE value of tcluster is correct }
      tcluster = ecluster;
      if !writecase(cluster_tracking_dict, tcluster) then
        {+EN} errmsg("Not able to write information about cluster %03d", tcluster);         {EN+}
        {{ES} errmsg("ES %03d");                                                  {ES}}
        {{FR} errmsg("Pas possible d'entrer les informations de la grappe %03d"); {FR}}
      endif;
    endif;
    close(cluster_tracking_dict);
  endif;

      if choice =   0 then {ESC} reenter; { escape from menu & allow entry of new cluster number }
  elseif choice =   3 then {A} transfer(choice);	{ Structure Check             }
  elseif choice =   4 then {B} csrun(choice); delb(){ Entry in batch              }
  elseif choice =   5 then {C} transfer(choice);	{ Verification                }
  elseif choice =   6 then {D} transfer(choice);	{ Backup Raw Data             }
  elseif choice =   7 then {---}                	{ do nothing                  }
  elseif choice =   8 then {E} csrun(choice);		{ Editing                     }
  elseif choice =   9 then {D} csrun(choice);		{ Modify the Data             }
  elseif choice =  10 then {G} transfer(choice);	{ Backup Final Data           }
  elseif choice =  11 then {---}               		{ do nothing                  }
  elseif choice =  12 then {H} transfer(choice);	{ Export to SPSS              }
  elseif choice =  13 then {---}               		{ do nothing                  }
  elseif choice =  14 then {I} csrun(choice);  		{ Enter GPS data              }
  elseif choice =  15 then {J} csrun(choice);		{ Modify GPS data             }
  elseif choice =  16 then {---}             		{ do nothing                  }
  elseif choice =  17 then {N} reenter;				{ get the new cluster number  }
  elseif choice =  18 then {V} csrun(choice);		{ print cluster tracking info }
  elseif choice =  19 then {P} transfer(choice);	{ field check tables          }
  elseif choice =  20 then {R} transfer(choice);	{ copy data file from operator}
  elseif choice =  21 then {S} transfer(choice);	{ copy veri file from operator}
  elseif choice =  22 then {QUIT}
    if fileexist(jstone) then
      filedelete(jstone);
    endif;
    stop(-1);       { quit the process }
  endif;
enddo;

PROC TRECVD
preproc
if startpos <> 1 and visualvalue($) <> 0 then
	advance TENTRYOPID;
endif;

PROC TRECVY
if startpos in 1:2 and TRECVD = 0 and TRECVM = 0 and $ = 0 then { Set to today's date }
  TRECVD = sysdate("DD");
  TRECVM = sysdate("MM");
  TRECVY = sysdate("YYYY");
endif;
recode TRECVY : TRECVM      	: TRECVD => err;
 2009 - 2012  : 1,3,5,7,8,10,12 : 1-31 => 0;
 2009 - 2012  : 4,6,9,11        : 1-30 => 0;
 2009 - 2011  : 2               : 1-28 => 0;
        2012  : 2               : 1-29 => 0;
              :                 :      => 1;
endrecode;
if err then
  {+EN} errmsg("Invalid date");  {EN+}
  {{ES} errmsg("ES");            {ES}}
  {{FR} errmsg("Date invalide"); {FR}}
  reenter
endif;
dd = (TRECVY*100 + TRECVM)*100 + TRECVD;
ss = sysdate("yyyymmdd");
if dd > ss then
  {+EN} errmsg("Date received impossible");  {EN+}
  {{ES} errmsg("ES");                        {ES}}
  {{FR} errmsg("Date reception impossible"); {FR}}
  reenter
endif;

PROC TNUMHHSEL
if $=0 then
  {+EN} errmsg ("Number of HHs selected for this cluster cannot equal zero");              {EN+}
  {{ES} errmsg ("ES"); {ES}}
  {{FR} errmsg ("Nombre de menages selectiones pur cette grappe ne peut pas egal a zero"); {FR}}
  reenter;
endif;
PROC TNUMHHQUEST
if TNUMHHQUEST <> TNUMHHSEL then
  {+EN} errmsg("Number of households (%02d) doesn't equal the number selected (%02d)", TNUMHHQUEST, TNUMHHSEL); {EN+}
  {{ES} errmsg("ES", TNUMHHQUEST, TNUMHHSEL); {ES}}
  {{FR} errmsg("Nombre de menages (%02d) n'est pas egal au nombre de menages selectionnes (%02d)", TNUMHHQUEST, TNUMHHSEL); {FR}}
	reenter;
endif;

PROC TNUMHHCOMP
if (TNUMHHQUEST < TNUMHHCOMP) then
  {+EN} errmsg("Number of completed (%02d) larger than the total number of households (%02d)", TNUMHHCOMP, TNUMHHQUEST); {EN+}
  {{ES} errmsg("ES", TNUMHHCOMP, TNUMHHQUEST); {ES}}
  {{FR} errmsg("Nombre de menages completes (%02d) plus grand que le total des menages (%02d)", TNUMHHCOMP, TNUMHHQUEST); {FR}}
	reenter;
endif;

PROC TNUMWMCOMP
if (TNUMWMQUEST < TNUMWMCOMP) then
  {+EN} errmsg("Number of completed (%02d) larger than the total number of women (%02d)", TNUMWMCOMP, TNUMWMQUEST); {EN+}
  {{ES} errmsg("ES", TNUMWMCOMP, TNUMWMQUEST); {ES}}
  {{FR} errmsg("Nombre de femmes completes (%02d) plus grand que le nombre total de femmes (%02d)", TNUMWMCOMP, TNUMWMQUEST); {FR}}
	reenter;
endif;

PROC TNUMMMQUEST

PROC TNUMMMCOMP
if (TNUMMMQUEST < TNUMMMCOMP) then
  {+EN} errmsg("Number of completed (%02d) larger than the total number of men (%02d)", TNUMMMCOMP, TNUMMMQUEST); {EN+}
  {{ES} errmsg("ES", TNUMWMCOMP, TNUMWMQUEST); {ES}}
  {{FR} errmsg("Nombre de femmes completes (%02d) plus grand que le nombre total de femmes (%02d)", TNUMWMCOMP, TNUMWMQUEST); {FR}}
	reenter;
endif;
PROC TNUMCHCOMP
if (TNUMCHQUEST < TNUMCHCOMP) then
  {+EN} errmsg("Number of completed (%02d) larger than the total number of children (%02d)", TNUMCHCOMP, TNUMCHQUEST); {EN+}
  {{ES} errmsg("ES", TNUMCHCOMP, TNUMCHQUEST); {ES}}
  {{FR} errmsg("Nombre d'enfants comletes (%02d) plus grand que le nombre total d'enfants (%02d)", TNUMCHCOMP, TNUMCHQUEST); {FR}}
	reenter;
endif;

if startpos = 1 then
	advance;
endif;

PROC TENTRYOPID
preproc
if startpos <> 2 then
	advance TVENTRYOPID;
endif;

postproc
if startpos <> 1 and $=0 then
  {+EN} errmsg ("Zero is not a valid operator ID, please reenter");                         {EN+}
  {{ES} errmsg ("ES");                                                                      {ES}}
  {{FR} errmsg ("Zero n'est pas valide pour l'ID de l'operateur, s'ilvous plait resaisir"); {FR}}
  reenter;
endif;
PROC TENTRYY
if startpos = 2 then
  if TENTRYD = 0 & TENTRYM = 0 & $ = 0 then { Set to today's date }
    TENTRYD = sysdate("DD");
    TENTRYM = sysdate("MM");
    TENTRYY = sysdate("YYYY");
  endif;
	{ Only check date when we are entering it }
	recode TENTRYY  :     TENTRYM     : TENTRYD => err;
	      2009-2012 : 1,3,5,7,8,10,12 : 1-31 => 0;
	      2009-2012 : 4,6,9,11        : 1-30 => 0;
	      2009-2011 : 2               : 1-28 => 0;
		       2012 : 2               : 1-29 => 0;
	                :                 :      => 1;
	endrecode;
	if err then
      {+EN} errmsg("Invalid date");  {EN+}
      {{ES} errmsg("ES");            {ES}}
	  {{FR} errmsg("Date invalide"); {FR}}
	  reenter
	endif;
	dd = (TENTRYY*100 + TENTRYM)*100 + TENTRYD;
	dr = (TRECVY*100 + TRECVM)*100 + TRECVD;
	ss = sysdate("yyyymmdd");
	if dd > ss or dd < dr then
      {+EN} errmsg("Date given before received date or after current date");                {EN+}
      {{ES} errmsg("ES");                                                                   {ES}}
      {{FR} errmsg("Date d'attribution avant date de reception or avant la date actuelle"); {FR}}
	  reenter
	endif;
	advance;
endif;

PROC TVENTRYOPID
{ if verification entry has been assigned [TVENTRYOPID <> 0]
  and veri data entry operator equals main data entry operator [TVENTRYOPID = TENTRYOPID]}
if startpos=3 then
  if $=0 then
    {+EN} errmsg ("Zero is not a valid operator ID, please reenter");                         {EN+}
    {{ES} errmsg ("ES");                                                                      {ES}}
    {{FR} errmsg ("Zero n'est pas valide pour l'ID de l'operateur, s'ilvous plait resaisir"); {FR}}
    reenter;
  
  elseif TVENTRYOPID = TENTRYOPID then
    {+EN} errmsg("Verification cannot be made by the same operator that did data entry");            {EN+}
    {{ES} errmsg("ES");                                                                              {ES}}
    {{FR} errmsg("Verification ne peut pas etre faite par le meme operateur qui a fait la saisie "); {FR}}
	reenter;
  endif;
endif;

PROC TVENTRYY
if startpos = 3 then
  if TVENTRYD = 0 & TVENTRYM = 0 & $ = 0 then { Set to today's date }
    TVENTRYD = sysdate("DD");
    TVENTRYM = sysdate("MM");
    TVENTRYY = sysdate("YYYY");
  endif;
	{ Only check date when we are entering it }
	recode TVENTRYY  :     TVENTRYM    : TVENTRYD => err;
	       2009-2012 : 1,3,5,7,8,10,12 : 1-31 => 0;
	       2009-2012 : 4,6,9,11        : 1-30 => 0;
	       2009-2011 : 2               : 1-28 => 0;
	            2012 : 2               : 1-29 => 0;
	                 :                 :      => 1;
	endrecode;
	if err then
      {+EN} errmsg("Invalid date");  {EN+}
      {{ES} errmsg("ES");            {ES}}
      {{FR} errmsg("Date invalide"); {FR}}
	  reenter
	endif;
	dd = (TVENTRYY*100 + TVENTRYM)*100 + TVENTRYD;
	dr = (TRECVY*100 + TRECVM)*100 + TRECVD;
	de = (TENTRYY*100 + TENTRYM)*100 + TENTRYD;
	ss = sysdate("yyyymmdd");
	if dd > ss or dd < dr or dd < de then
      {+EN} errmsg("Date given before received date or data entry date, or after current date");                  {EN+} 
      {{ES} errmsg("ES");                                                                                         {ES}} 
	  {{FR} errmsg("Date d'attribution avant date de reception ou  avant date de saisie ou avant date actuelle"); {FR}}
	  reenter
	endif;
endif;
